#include "particle.h"
#include "ofMain.h"


//------------------------------------------------------------
particle::particle(){
	setInitialCondition(0,0,0,0);
	damping = 0.15f;
	size = 10;
	angle =0;
}

//------------------------------------------------------------
void particle::resetForce(){
    // we reset the forces every frame
    frc.set(0,0);
}

//------------------------------------------------------------
void particle::addForce(float x, float y){
    // add in a force in X and Y for this frame.
    frc.x = frc.x + x;
    frc.y = frc.y + y;
}

//------------------------------------------------------------
void particle::addDampingForce(){
	
	// the usual way to write this is  vel *= 0.99
	// basically, subtract some part of the velocity 
	// damping is a force operating in the oposite direction of the 
	// velocity vector
	
    frc.x = frc.x - vel.x * damping;
    frc.y = frc.y - vel.y * damping;
}

//------------------------------------------------------------
void particle::setInitialCondition(float px, float py, float vx, float vy){
    pos.set(px,py);
	prevPos.set(0,0);
	vel.set(vx,vy);
}

//------------------------------------------------------------
void particle::update(){	
	vel = vel + frc;
	pos = pos + vel;
	
	ofPoint temp;
	temp.x = pos.x;
	temp.y = pos.y;
	pts.push_back(temp);
	if(pts.size() > 100){ 
		pts.erase(pts.begin());
		//ofSetColor(ofRandom(100, 255), 0,0);
	}
}

//------------------------------------------------------------
void particle::draw(){

ofFill();
	ofPushMatrix();
		ofTranslate(pos.x, pos.y,0);
		ofRotateZ(angle * RAD_TO_DEG + 90);
		//ofLine(0,0,  size + 20, size + 20 );
		ofNoFill();
		ofRect(0, 0, 10, 5); // after ofTranslate, the shapes need to be drawn from (0, 0)
	
		//ofSetColor(255,0,0);	
	
	
		ofNoFill();
		ofBeginShape();
		for (int i = 0; i < pts.size(); i++){
			ofVertex(pts[i].x-pos.x, pts[i].y-pos.y);  //in order to follow ofRect, 
		}												//ofVertext needs to be be drawn at the end of ofRect
	
		ofEndShape();
	
	ofPopMatrix();
	
	
}


//------------------------------------------------------------

		
	void particle::xenoToPoint(float catchX, float catchY){
	

		float catchUpSpeed = 0.02;

	pos.x = catchUpSpeed * catchX + (1-catchUpSpeed) * pos.x; 
	pos.y = catchUpSpeed * catchY + (1-catchUpSpeed) * pos.y; 

		float dx = pos.x - prevPos.x;
		float dy = pos.y - prevPos.y;

		angle = atan2(dy, dx);

		prevPos.x = pos.x;
		prevPos.y = pos.y;
	
	}
//_________________________________________
void particle::bounceOffWalls(){
	
	// sometimes it makes sense to damped, when we hit
	bool bDampedOnCollision = true;
	bool bDidICollide = false;
	
	// what are the walls
	float minx = 0;
	float miny = 0;
	float maxx = ofGetWidth();
	float maxy = ofGetHeight();
	
	if (pos.x > maxx){
		pos.x = maxx; // move to the edge, (important!)
		vel.x *= -1;
		bDidICollide = true;
	} else if (pos.x < minx){
		pos.x = minx; // move to the edge, (important!)
		vel.x *= -1;
		bDidICollide = true;
	}
	
	if (pos.y > maxy){
		pos.y = maxy; // move to the edge, (important!)
		vel.y *= -1;
		bDidICollide = true;
	} else if (pos.y < miny){
		pos.y = miny; // move to the edge, (important!)
		vel.y *= -1;
		bDidICollide = true;
	}
	
	if (bDidICollide == true && bDampedOnCollision == true){
		vel *= 0.3;
	}
	
}
